%SETUPGRIND - Setup GRIND
%   Setup script for GRIND, adds the grind system directory to
%   the default path. This file needs only to be run once after
%   installing GRIND.
%
%   SETUPGRIND -U  - uninstalls GRIND (removes no files)
%   SETUPGRIND -S  - adds "setupgrind" to startup.m file (use this option if the
%                    matlab path cannot be changed permanently)
%
%   See also:
%   Installing
function setupgrind(option)
doanyway = 0;
uninstall = 0;
addstartup = 0;
if nargin == 1
   if strncmpi(option, '-d', 2)
      doanyway = 1;
   end;
   if strncmpi(option, '-u', 2)
      uninstall = 1;
   end;
   if strncmpi(option, '-s', 2)
      addstartup = 1;
   end;
end;
k = which('grind.m');
if ~isempty(k) & ~doanyway
   thegrindpath = fileparts(k);
   if uninstall
      try
         [release, system] = getrelease;
      catch
         release = 11;
         system = 'matlab';
      end;
      isoctave = strcmpi(system, 'octave');
      rmpath(thegrindpath);
      if release == 11
         rmpath(fullfile(thegrindpath, 'R11'))
      end;
      if release <= 12
         rmpath(fullfile(thegrindpath, 'R12'))
      end;
      if isoctave
         rmpath(fullfile(thegrindpath, 'octave'))
      end;
      try
         savepath; %adds permanently
      catch
         path2rc;
      end;
      fprintf('Grind uninstalled from %s (no files removed)\n', thegrindpath);
   else
      fprintf('Grind was already installed in %s\n', thegrindpath);
      if addstartup
         doaddstartup;
      end;
   end;
   return;
elseif uninstall
   fprintf('Grind was not installed\n');
   return;
end;
try
   try
      mroot = OCTAVE_PATH;
   catch
      mroot = matlabroot;
   end;
catch
   mroot = '';
end;
thegrindpath=fullfile(mroot,'work','grind','sys');
k = which(fullfile(thegrindpath, 'grind.m'));
if isempty(k)
   thegrindpath = fullfile(pwd, 'sys');
end;
addpath(thegrindpath);
[release, system] = getrelease;
isoctave = strcmpi(system, 'octave');
if release == 11
   addpath(fullfile(thegrindpath, 'R11'));
end;
if release <= 12
   addpath(fullfile(thegrindpath, 'R12'))
end;
if isoctave
   addpath(fullfile(thegrindpath, 'octave'));
end;
try
   savepath; %adds permanently
catch
   path2rc;
end;
w = 0;
if ~canwriteto(grindpath)
   disp(['WARNING: no permission to write to ' grindpath]);
   disp('If you have permission in another directory it is possible to run GRIND');
   disp('Preferrably set write permission to the full GRIND directory.');
   disp(' ');
   w = 1;
end;
p=fullfile(mroot,'toolbox','local');
if ~canwriteto(p)
   disp(['WARNING: the path cannot be stored as there is no write permission in ' p]);
   disp('Each session GRIND needs to be reinstalled (or the permission should be changed).');
   disp(' ');
   w = 1;
end;
if addstartup
   doaddstartup;
end;
if w
   disp('GRIND installed, but there are some problems (see above)');
else
   disp('Grind installed successfully');
end;
function doaddstartup
thegrindpath=grindpath;
u = userpath;
if length(u) < 4  %R11 gives not the workdirectory
   u = fullfile(matlabroot, 'work');
end;
f = findstr(';', u);
if ~isempty(f)
   u = u(1:f(1) - 1);
end;
u = fullfile(u, 'startup.m');
if exist(u, 'file') == 2
   lines = {};
   fid = fopen(u, 'r');
   hassetup = 0;
   while ~feof(fid)
      line = fgetl(fid);
      lines = {lines, line};
      if ~isempty(findstr('setupgrind', line))
         hassetup = 1;
      end;
   end;
   fclose(fid);
   if ~hassetup
      fid = fopen(u, 'a');
      fprintf(fid,'\ncd ''%s''\nsetupgrind\n',thegrindpath(1:end-4));
      fclose(fid);
      disp('setupgrind added to startup.m');
   else
       disp('setupgrind was already in startup.m');
   end;
else
   fid = fopen(u, 'w');
   fprintf(fid,'\ncd ''%s''\nsetupgrind\n',thegrindpath(1:end-4));
   disp('startup.m created');
   fclose(fid);
end;
function ok = canwriteto(path)
ok = 0;
oldpath = pwd;
try
   cd(path);
   fid=fopen('t.tmp','wt');
   ok =fid >= 0;
   if ok
      fclose(fid);
   end;
   delete('t.tmp');
   cd(oldpath);
catch
   cd(oldpath);
end;

