%BACKW - Simulate the model backwards
%   Run the model with negated right-hand sides (=backwards). 
%   For difference equations this is more complicated due to the 
%   discrete steps. Therefore a local optimizer is used to find 
%   the next step back in time.
%   This way you can find unstable equilibria (see also: FINDEQ) or a separatrix.
%   (see: PERTURB).
%
%   Usage:
%   BACKW - run backwards with g_grind.ndays time steps (see also SIMTIME for setting the simulation duration).
%   BACKW N - run backwards for N steps
%
%   See also RU, SIMTIME, SOLVER, PERTURB, ADDMODE

%   Copyright 2011 WUR
%   Revision: 1.0.7 $ $Date: 27-Oct-2011 17:17:07 $
function backw(ndays,dotrunc)
global g_grind;
i_parcheck;
if nargin == 0
   ndays = g_grind.ndays;
else
   ndays = i_checkstr(ndays);
end;
if nargin<2
   dotrunc=1;
end;
g_grind.solver.backwards =  ~g_grind.solver.backwards;
oldtrunc=g_grind.truncate;
g_grind.truncate=dotrunc;
try
   ru(ndays)
   g_grind.solver.backwards = ~g_grind.solver.backwards;
   g_grind.truncate=oldtrunc;
catch
   err=lasterror;
   g_grind.solver.backwards = ~g_grind.solver.backwards;
   g_grind.truncate=oldtrunc;
   rethrow(err);
end;
