%CLIPBOARD - Copy matrix to clipboard
%   Copy a matrix or a cell array (containing only values or character strings) to the Windows clipboard, 
%   as text tab delimited.
%
%   Usage:
%   CLIPBOARD(g_t) - Copy the variable g_t to the clipboard.
%   CLIPBOARD g_Y - Copy variable g_Y to clipboard.
%
%   See also PASTE

%   Copyright 2011 WUR
%   Revision: 1.0.7 $ $Date: 27-Oct-2011 17:17:08 $
function res=clipboard(A,par)
direct = pwd;
if ~strcmp(computer,'PCWIN')&(nargin==1)
    clipboard('copy',A);
end;    
if (nargin==2)|(nargout==1)|strcmp(A,'paste') 
    %in newer versions of MATLAB clipboard is added this function has 2 arguments
   d=which('clipboard','-all'); %find overloaded clipboard.m
   if length(d)>1 %run the MATLAB version of clipboard
       cd(fileparts(d{2}));
       if (nargout==1)
          res=clipboard('paste');
       elseif (nargin==1)
          clipboard(A)
      else
          clipboard(A,par);
       end;
       cd(direct);
       return;
   else
       if strcmp(A,'paste')
          paste(par);
       else
          A=par;
       end;           
   end;    
end;
cd(grindpath);
if ischar(A)
   b = evalin('base', A);
   dlmwrite1('dlm.txt',b,'\t');
else
   dlmwrite1('dlm.txt',A,'\t');
end;
!dlmclp
delete('dlm.txt');
cd(direct);

%adapted dlmwrite function (it writes 0's also)
function dlmwrite1(filename, m, dlm)
NEWLINE = sprintf('\n');
r = 0;
c = 0;
% delimiter defaults to Comma for CSV
dlm = sprintf(dlm); % Handles special characters.
% open the file
if strncmp(computer, 'MAC', 3)
   fid = fopen(filename , 'wt');
else
   fid = fopen(filename , 'wb');
end

if fid == (-1),
   error(['Could not open file ' filename]);
end

% dimensions size of matrix
[br, bc] = size(m);


% start with offsetting row of matrix
for i = 1:r
   for j = 1:bc + c - 1
      fwrite(fid, dlm, 'uchar');
   end
   fwrite(fid, NEWLINE, 'char');
end
% start dumping the array, for now number format float
if iscell(m)
   for i = 1:br
      for j = 1:c
         fwrite(fid, dlm, 'uchar');
      end
      for j = 1:bc
         if ~ischar(m{i, j})
            str = num2str(m{i, j});
            fwrite(fid, str, 'uchar');
         else
            fwrite(fid, m{i,j},'uchar');
         end;
         if (j < bc)
            fwrite(fid, dlm, 'uchar');
         end
      end
      if i < br
         fwrite(fid, NEWLINE, 'char'); % this may  \ r \ n for DOS
      end;
   end
else
   for i = 1:br
      % start with offsetting col of matrix
      for j = 1:c
         fwrite(fid, dlm, 'uchar');
      end
      for j = 1:bc
         %       if(m(i,j) ~= 0)
         str = num2str(m(i, j));
         fwrite(fid, str, 'uchar');
         %      end
         if (j < bc)
            fwrite(fid, dlm, 'uchar');
         end
      end
      if i < br
         fwrite(fid, NEWLINE, 'char'); % this may  \ r \ n for DOS
      end;
   end
end;
% close files
fclose(fid);
