%DOWNCELLS - shift down the elements of a matrix/vector
%  Efficient function to shift the elements of a matrix one row down. This
%  function is used to access neighbors of a matrix or vector state variable. 
%  Optionally, the first row is neighboring to the last row. 
%
%  Usage:
%  DOWNCELLS(N,BORDER) - shift the matrix N 1 position down. If BORDER is 1 the first row 
%  borders to itself, if BORDER is 0 the first row borders the last. 
%  DOWNCELLS(N,2,VALUE) - if BORDER is 2 the boundary condition VALUE is used at the border.
% 
%  See also MODEL, RIGHTCELLS, LEFTCELLS, UPCELLS

%   Copyright 2011 WUR
%   Revision: 1.0.7 $ $Date: 27-Oct-2011 17:17:08 $
function Result=downcells(N,bordered,aValue)
if nargin<2
   bordered=1;
end;
if (bordered==2)&(nargin<3)
   error('Downcells: No boundary condition given');
end;
s=size(N,1);
if bordered==1
   Result=[N(1,:);N(1:s-1,:)];
elseif bordered==0
   Result=[N(s,:);N(1:s-1,:)];
elseif bordered==2
   Result=[ones(1,size(N,2))+aValue;N(1:s-1,:)];
end;


