%PAR - Show parameters and model
%   Show the model equations and the values of parameters
%
%   See also STAT

%   Copyright 2011 WUR
%   Revision: 1.0.7 $ $Date: 27-Oct-2011 17:17:09 $
function p = par(full, p1)
global g_grind;
if nargin == 0
   full = '';
elseif ~isempty(full) & strcmp(full, '?')
   e = par('');
   [str,ok]=listdlg('liststring',e,'Name','Parameters',...
      'PromptString','Select parameter','SelectionMode',...
      'single','uh',40,'ffs',10,'fus',10);
   if ok
      f = strfind(e{str}, '=');
      if ~isempty(f)
         parname = strtrim(e{str}(1:f(1) - 1));
         val = strtrim(e{str}(f(1) + 1:length(e{str}) - 1));
         val=inputdlg(['Change parameter ' parname],'Change parameter',1,{val});
         assignin('base', parname, str2num(val{1})); %#ok
      end;
   end;
elseif strcmp(full, '-maxparlen')
   p = 1;
   if isfield(g_grind, 'model') 
   for i = 1:length(g_grind.pars)
      if length(g_grind.pars{i}) > p
         p = length(g_grind.pars{i});
      end;
   end;
   end;
   return;
elseif strncmpi(full, '-d', 2)
   p = findparcomm(p1);
   if ~isempty(p)
      f = strfind(p,',');
      if ~isempty(f)
         p = p(1:f(end) - 1);
      end;
   end;
   return;
elseif strncmpi(full, '-u', 2)
   p = findparcomm(p1);
   if ~isempty(p)
      f = strfind(p,',');
      if isempty(f)
         p = '';
      else
         p = p(f(end) + 1:end);
      end;
   end;
   return;
end;
if nargout == 0
   % disp(' ');
   if strcmpi(full, 'modelonly')
      disp('Current model');
   elseif strcmpi(full, '-size')
      disp('Sizes of parameters/variables');
      if g_grind.statevars.vector
         dispsiz(g_grind.statevars.vectnames);
      else
         dispsiz(g_grind.statevars.names);
      end;
      dispsiz(g_grind.pars);
      return
   else
      disp('Current model and parameters');
   end;
   if ~isfield(g_grind, 'model') | isempty(g_grind.model)
      disp('No model selected');
   else
      for i = 1:size(g_grind.model, 2)
         disp(char(g_grind.model{i}));
      end;
   end;
end;
maxparlen = par('-maxparlen');

if isfield(g_grind, 'pars')&~strcmpi(full, 'modelonly')
   pp = cell(1, size(g_grind.pars, 2));
   for i = 1:size(g_grind.pars, 2)
      ppar = evalin('base', g_grind.pars{i});
      if (size(ppar, 1) > 1) | (size(ppar, 2) > 1)
         s=[g_grind.pars{i} ' = [' ];
         siz = size(ppar);
         sh = 0;
         if ~strcmpi(full, 'full')
            if (siz(1) > 10)
               siz(1) = 10;
               sh = 1;
            end;
            if siz(2) > 10
               siz(2) = 10;
               sh = 1;
            end;
         end;
         for j = 1:siz(1)
            for k = 1:siz(2);
               s = [s sprintf('%g, ',ppar(j,k))];
            end;
            if j < size(ppar, 1)
               s(length(s) - 1) = ';';
               s = [s sprintf('...\n    ')];
            end;
         end
         s(length(s) - 1) = ']';
         s(length(s)) = ';';
         if sh
            s=[s sprintf('\n') '%(first 10 elements shown)'];
         end;
         pp{i} = s;
      elseif ~isempty(ppar)
         pp{i}=sprintf(['%-' num2str(maxparlen) 's = %0.5g;'], g_grind.pars{i}, ppar);
      else
         pp{i}=sprintf(['%-' num2str(maxparlen) 's = [];'], g_grind.pars{i});
      end;
      descr = par('-d', g_grind.pars{i});
      if ~isempty(descr)
         pp{i}=sprintf(['%-' num2str(maxparlen+25-8) 's %%%s,%s'],pp{i},descr,par('-u',g_grind.pars{i}));
      end;
   end;
   if g_grind.solver.hasevents
      disp('Discrete events');
      setevent  '-list';
   end;
   if nargout == 1
      p = pp;
   else
      if isfield(g_grind, 'pars')
         for i = 1:size(g_grind.pars, 2)
            disp(pp{i});
         end;
      end;
      if ~isempty(g_grind.externvars)
         defextern;
      end;
      i_parcheck(1);
   end;
end;
function dispsiz(v)
for i = 1:size(v, 2)
   siz = evalin('base', sprintf( 'size(%s)',v{i}));
   fprintf('%s = [%dx%d]\n', v{i}, siz);
end;
function parcomm = findparcomm(p);
global g_grind;
n = 0;
parcomm = '';
lenp = length(p);
for i = 1:length(g_grind.commands);
   s = trimchar(g_grind.commands{i}, ' ');
   f1 = strfind(s, '%');
   if ~isempty(f1) & (f1 < length(s))
      if strncmpi(p, trimchar(s, ' '),lenp)
         if (length(s) > lenp) & ~isempty(strfind(s(lenp + 1), '( ='))
            parcomm = s(f1 + 1:end);
            return;
         end;
      end;
   end;
end;


