%SOLVER - Set the solver and settings.
%   For difference equations there is only one method available.
%   Enter the following information:
%   - Solver (rk4 Euler ode45 ode23 ode115 ode15S ode23S ode23T ode23TB)
%   - settings for the solver, see: MATLAB description or RK4/ EULER
%
%   See also RK4, EULER, ODE45, ODE23, ODE115, ODE15S, ODE23S, ODE23T, ODE23TB

%   Copyright 2011 WUR
%   Revision: 1.0.7 $ $Date: 27-Oct-2011 17:17:09 $
function res=solver(solv, p1, p2)
global g_grind;
%i_parcheck;
if nargin==1
   if strcmpi(solv,'step')
      res=g_grind.solver.opt.MaxStep;
      if isempty(res)
         error(sprintf('No step size defined for the current solver ("%s")\n',g_grind.solver.name));
      end;
      return;
   elseif strcmpi(solv,'name')
      res=g_grind.solver.name;
      return
   end;
end;
if g_grind.solver.isdiffer
   disp('For difference equations, there is only one solver method');
elseif g_grind.solver.haslag
   disp('For delay differential equations (DDEs), there is only one solver method (DDE23)');
else
   solverlist = {'rk4','euler','ode45','ode23','ode113','ode15S', 'ode23S', 'ode23T', 'ode23TB'};
   if nargin > 0
      if solv=='?'
         if isempty(g_grind.solver.opt.MaxStep)
            fprintf('solver %s %g %g\n',g_grind.solver.name,g_grind.solver.opt.RelTol,g_grind.solver.opt.AbsTol);
         else
            fprintf('solver %s %g\n',g_grind.solver.name,g_grind.solver.opt.MaxStep);
         end;
         return;
      end;      
      n = [];
      for i = 1:length(solverlist)
         if strcmpi(solverlist{i}, solv)
            n = i;
         end;
      end;
      if ~isempty(n)
         g_grind.solver.name = solverlist{n};
         if (n <= 2) & (nargin > 1)
            g_grind.solver.opt.MaxStep = i_checkstr(p1);
         elseif nargin == 3
            g_grind.solver.opt.RelTol = i_checkstr(p1);
            g_grind.solver.opt.AbsTol = i_checkstr(p2);
         end;
      end;
   else
      n = 1;
      for i = 1:length(solverlist)
         if strcmpi(solverlist{i}, g_grind.solver.name)
            n = i;
         end;
      end;
      i = listdlg('ListString', solverlist, ...
         'SelectionMode', 'single',...
         'PromptString', 'Select a solver',...
         'InitialValue', n);
      if ~isempty(i)
         g_grind.solver.name = solverlist{i};
         if i <= 2
            if isempty(g_grind.solver.opt.MaxStep)
               answer = {'0.3'};
            else
               answer = {num2str(g_grind.solver.opt.MaxStep)};
            end;
            answer = inputdlg({'Step size'},'Solver options',1,answer);
            g_grind.solver.opt.MaxStep = i_checkstr(answer{1});
         else
            g_grind.solver.opt.MaxStep = [];
            prompt = {'Relative tolerance' , ...
               'Absolute tolerance'};
            answer = {'1e-3','1e-6'};
            if ~isempty(g_grind.solver.opt.RelTol)
               answer{1} = num2str(g_grind.solver.opt.RelTol);
            end;
            if ~isempty(g_grind.solver.opt.AbsTol)
               answer{2} = num2str(g_grind.solver.opt.AbsTol);
            end;
            answer = inputdlg(prompt, 'Solver options', 1, answer);
            if ~isempty(answer)
               g_grind.solver.opt.RelTol = i_checkstr(answer{1});
               g_grind.solver.opt.AbsTol = i_checkstr(answer{2});
            end;
         end
      end;
   end
end
